/* forw.h - code to deal with forward references */

typedef struct code
	{
		int done;		/* has it been evaluated yet	*/
					/* better to delete from list	*/
					/* but this is easier to write!	*/
					/* next version / manyana ;-)	*/
		int address;
		int scope;
		EXPR_NODE *value;
		char *filename;
		int   lineno;
		struct code *next;
	} CODE_ADDR;

/* ... The End ... */

