typedef union {
	int	integer;
	char	string[128];
	EXPR_NODE *exprstruct;
} YYSTYPE;
#define	NUM	258
#define	ID	259
#define	STRING	260
#define	ORG	261
#define	BLOCK	262
#define	END	263
#define	DATA	264
#define	WARN	265
#define	ERROR	266
#define	THEEOF	267
#define	ASSIGN	268
#define	COLON	269
#define	ADD	270
#define	SUB	271
#define	MUL	272
#define	DIV	273
#define	MOD	274
#define	SHL	275
#define	SHR	276
#define	LAND	277
#define	LOR	278
#define	BNOT	279
#define	LNOT	280
#define	BAND	281
#define	BOR	282
#define	BXOR	283
#define	EQ	284
#define	NEQ	285
#define	GT	286
#define	GEQ	287
#define	LS	288
#define	LEQ	289
#define	LPAR	290
#define	RPAR	291
#define	COMMA	292
#define	PERIOD	293
#define	UMINUS	294


extern YYSTYPE yylval;
