/* macstuff.h macro definition stuff */

#include <stdio.h>
#include <string.h>

typedef struct tok
	{
		int         type;
		int         intval;
		char       *strval;
		struct tok *next;
	} TOKEN;

typedef struct mac
	{
		char	   *name;
		TOKEN	   *tokens;
		struct mac *next;
	} MAC_NODE;

#define MAX_ARGS 10		/* n.b. fixed values to prevent inf recursion */
#define MAX_MACRO_DEPTH	10	/*      and general stupidity		      */

/* ... The End ... */
