/* expr.h */

#include <stdio.h>
#include <string.h>

typedef struct exprnode
	{
		int   type;
		int   value;
		char *idptr;
		struct exprnode *next;
	} EXPR_NODE;

#define EXPR_NUM	0
#define EXPR_OP		1
#define EXPR_ID		2

#define MAX_STACK 20

/* ... The End ... */

