/* picdis.h  - constants and includes	*/
/* version 0.1				*/
/* (c) I.King 1994			*/

#include <stdio.h>
#include <ctype.h>

#define PIC12	0
#define PIC14	1

#define LOWERCASE	0
#define UPPERCASE	1

#define MAXPICSIZE 8192

/* ... The End ... */
