/* symbol.h */

#include <stdio.h>
#include <string.h>

typedef struct sym
	{
		int   type;
		char *name;
		int   value;
		int   defined;
		int   scope;
		struct sym *next;
	} SYMBOL;

#define UNDEFINED	0
#define DEFINED		1

#define UNKNOWNDEF	0
#define VARDEF		1
#define LABDEF		2

/* ... The End ... */

