/* pu_defs.h					*/
/* definitions for the pic config read routines	*/
/* I.King 1994					*/

typedef struct picdefn
	{
		int		pictype;
		int		picid[4];
		float		clock;
		int		osctype;
		int		cp_fuse;
		int		wd_fuse;
		int		pu_fuse;
	} PICDEFN;

#define PU_OK	1
#define PU_FAIL	0


/* ... The End ... */
