/* emit.h */

#define MAX_PIC_SIZE	65536	/* this is the maximum possible for a
				   normal 17Cxx series processor */
#define DEFAULT_EMPTY_MEM 0xffff

#define ASMRAW		0
#define ASMIHX16	1

/* ... The End ... */

